--------------------------------------------------------------------------------
-- Dynamite On Paper Plane by holzchopf
-- v1.1 - 03 Oct 2011	- Fixed sound glitch
--------------------------------------------------------------------------------

-- setup
if holzchopf==nil then holzchopf={} end
holzchopf.dynamitepaperplane={}
holzchopf.dynamitepaperplane.plane={}

-- load ressources
holzchopf.dynamitepaperplane.gfx_wpn=loadgfx("holzchopf/dynamiteonpaperplane.bmp")
setmidhandle(holzchopf.dynamitepaperplane.gfx_wpn)
holzchopf.dynamitepaperplane.sfx_throw=loadsfx("throw.ogg")
holzchopf.dynamitepaperplane.sfx_fuse=loadsfx("fuse.wav")

--------------------------------------------------------------------------------
-- Weapon
--------------------------------------------------------------------------------
holzchopf.dynamitepaperplane.id=addweapon("holzchopf.dynamitepaperplane","Dynamite on Paper Plane",holzchopf.dynamitepaperplane.gfx_wpn,0)

function holzchopf.dynamitepaperplane.draw()
	if weapon_shots==0 then
		-- draw plane in hand if not fired
		setblend(blend_alpha)
		setalpha(1)
		setcolor(255,255,255)
		drawinhand(holzchopf.dynamitepaperplane.gfx_wpn,6,0)
		-- HUD crosshair
		hudcrosshair(6,3)
		hudtimer(3,1,10)
		-- HUD chargebar
		if weapon_charge>0 then
			hudchargebar(weapon_charge,100)
		end
	end
end

function holzchopf.dynamitepaperplane.attack(attack)
	if weapon_shots>0 then
		return
	end
	-- charge
	if attack==1 then
		weapon_charge=weapon_charge+1
	end
	-- release
	if (attack==0 and weapon_charge>0) or weapon_charge>=100 then
		-- use weapon
		useweapon(0)
		endturn()
		weapon_shots=weapon_shots+1
		playsound(holzchopf.dynamitepaperplane.sfx_throw)
		-- create projectile
		id=createprojectile(holzchopf.dynamitepaperplane.plane.id)
		projectiles[id]={}
		projectiles[id].ignore=playercurrent()
		projectiles[id].timer=getframe()+weapon_timer*50
		-- pose
		projectiles[id].x=getplayerx(0)+6*getplayerdirection(0)
		projectiles[id].y=getplayery(0)+3
		projectiles[id].rot=getplayerrotation(0)
		projectiles[id].w=0
		projectiles[id].dir=getplayerdirection(0)
		-- speed
		projectiles[id].sx=math.sin(math.rad(getplayerrotation(0)))*weapon_charge*0.1
		projectiles[id].sy=-math.cos(math.rad(getplayerrotation(0)))*weapon_charge*0.1
	end
end

--------------------------------------------------------------------------------
-- Plane
--------------------------------------------------------------------------------
holzchopf.dynamitepaperplane.plane.id=addprojectile("holzchopf.dynamitepaperplane.plane")

function holzchopf.dynamitepaperplane.plane.draw(id)
	-- draw projectile
	setblend(blend_alpha)
	setcolor(255,255,255)
	setalpha(1)
	setscale(projectiles[id].dir,1)
	setrotation(projectiles[id].rot)
	drawimage(holzchopf.dynamitepaperplane.gfx_wpn,projectiles[id].x,projectiles[id].y)
	-- draw arrow if out of screen
	outofscreenarrow(projectiles[id].x,projectiles[id].y)
end

function holzchopf.dynamitepaperplane.plane.update(id)
	-- velocity
	local vx=projectiles[id].sx-getwind()*20
	local vy=projectiles[id].sy
	local v=math.sqrt(vx*vx+vy*vy)
	local vex=vx/v
	local vey=vy/v
	-- orientation
	local a=math.rad(projectiles[id].rot)
	local ox=math.sin(a)
	local oy=-math.cos(a)
	-- orientation / movement relation
	local v_eu=vex*ox+vey*oy
	local v_ev=vey*ox-vex*oy
	local v_u=v_eu*v*projectiles[id].dir
	local v_v=v_ev*v
	-- torque due to load (rotation dependent)
	local m_l=angledelta(projectiles[id].rot,180)*0.2
	-- torque due to wind resistance (forward speed dependent)
	local m_s=-4*v_eu^2*v_u
	-- apply torques
	projectiles[id].w=projectiles[id].w*0.90+0.1*(m_l+m_s)
	projectiles[id].rot=projectiles[id].rot+projectiles[id].w
	-- gravity force
	local f_g=getgravity()
	-- braking force due to drag
	local f_b=-0.0001*v_u^2
	-- lifting force due to drag
	local f_l=-0.5*v_ev^2*v_v
	-- uplift
	local f_u=-0.02*v_eu^2
	-- apply forces
	projectiles[id].sx=projectiles[id].sx+ox*(f_b)-oy*(f_l+f_u)
	projectiles[id].sy=projectiles[id].sy+oy*(f_b)+ox*(f_l+f_u)+f_g
	-- move (in substep loop for optimal collision precision)
	msubt=math.ceil(math.max(math.abs(projectiles[id].sx),math.abs(projectiles[id].sy))/5)
	msubx=projectiles[id].sx/msubt
	msuby=projectiles[id].sy/msubt
	for i=1,msubt do
		local tempx=projectiles[id].x
		local tempy=projectiles[id].y
		projectiles[id].x=projectiles[id].x+msubx
		projectiles[id].y=projectiles[id].y+msuby
		-- collision
		if collision(col5x5,projectiles[id].x,projectiles[id].y,1,1,1,projectiles[id].ignore)==1 then
			-- define angle
			local mx=0
			local my=0
			for iy=-2,2 do
				for ix=-2,2 do
					if collision(col1x1,projectiles[id].x+ix,projectiles[id].y+iy,1,1,1,projectiles[id].ignore)==1 then
						mx=mx+ix
						my=my+iy
					end
				end
			end
			local m=math.sqrt(mx*mx+my*my)
			-- repel
			projectiles[id].sx=-mx/m --*math.abs(projectiles[id].sx)
			projectiles[id].sy=-my/m --*math.abs(projectiles[id].sy)		
			-- reset
			projectiles[id].x=tempx
			projectiles[id].y=tempy
			break
		else
			projectiles[id].ignore=0
		end
		-- water
		if (projectiles[id].y+20)>getwatery() then
			-- fx
			particle(p_waterhit,projectiles[id].x,projectiles[id].y)
			playsound(sfx_hitwater1)
			stopchannel(0)
			-- free projectile
			freeprojectile(id)
			return
		end
	end
	-- sfx
	if channelplaying(0)==0 then playsound(holzchopf.dynamitepaperplane.sfx_fuse,0) end
	-- explode
	if getframe()>=projectiles[id].timer then
		-- cause damage
		arealdamage(projectiles[id].x,projectiles[id].y,125,65)
		-- destroy terrain
		terrainexplosion(projectiles[id].x,projectiles[id].y,55,1)
		-- crater
		grey=math.random(0,40)
		if math.random(0,1)==1 then
			terrainalphaimage(gfx_crater150,projectiles[id].x,projectiles[id].y,math.random(6,9)*0.1,grey,grey,grey)
		else
			terrainalphaimage(gfx_crater175,projectiles[id].x,projectiles[id].y,math.random(6,9)*0.1,grey,grey,grey)
		end
		stopchannel(0)
		-- free projectile
		freeprojectile(id)
	end
	-- gfx
	local f=getframe()
	if f%4==0 then
		particle(p_smoke,projectiles[id].x-ox*8,projectiles[id].y-oy*8)
		particlespeed(math.random(-2,2)*0.1+getwind()*20,math.random(-6,-3)*0.1)
		particlefadealpha(0.01)
		particle(p_spark,projectiles[id].x-ox*8,projectiles[id].y-oy*8)
	end
	if f%6==0 then
		particle(p_lightpuff,projectiles[id].x-ox*8,projectiles[id].y-oy*8)
		particlespeed(math.random(-2,2)*0.1+getwind()*20,math.random(-5,-2)*0.1)
		particlefadealpha(0.04)
	end
	-- scroll to projectile
	scroll(projectiles[id].x,projectiles[id].y)
end
